@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: pr_o.ml,v 6.54 2012-01-09 14:22:21 deraugla Exp $ *)
+(* $Id: pr_o.ml,v 6.62 2012-03-18 09:00:42 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 #directory ".";
@@ -245,6 +245,17 @@ value strip_heading_spaces s =
     else String.sub s i (String.length s - i)
 ;
 
+value strip_char c s = do {
+  let s = String.copy s in
+  loop 0 0 where rec loop i j =
+    if i = String.length s then String.sub s 0 j
+    else if s.[i] = '_' then loop (i + 1) j
+    else do {
+      s.[j] := s.[i];
+      loop (i + 1) (j + 1)
+    }
+};
+
 (* expression with adding the possible comment before *)
 value comm_expr expr pc z =
   let loc = MLast.loc_of_expr z in
@@ -1039,8 +1050,12 @@ EXTEND_PRINTER
 *)
                   e)
       | <:expr< let module $uid:s$ = $me$ in $e$ >> ->
-          pprintf pc "@[<a>let module %s =@;%p@ in@]@ %p" s module_expr me
-            curr e
+          if pc.dang = ";" then
+            pprintf pc "(@[<a>let module %s =@;%p@ in@]@ %p)" s module_expr me
+              curr e
+          else
+            pprintf pc "@[<a>let module %s =@;%p@ in@]@ %p" s module_expr me
+              curr e
       | <:expr:< while $e1$ do { $list:el$ } >> ->
           pprintf pc "@[<a>@[<a>while@;%p@ do@]@;%p@ done@]" curr e1
             (hvlistl (semi_after expr) curr) el
@@ -1237,6 +1252,10 @@ EXTEND_PRINTER
       | <:expr< (module $me$) >> ->
           pprintf pc "(module %p)" module_expr me
       | <:expr< $int:s$ >> | <:expr< $flo:s$ >> ->
+          let s =
+            if flag_compatible_old_versions_of_ocaml.val then strip_char '_' s
+            else s
+          in
           if String.length s > 0 && s.[0] = '-' then pprintf pc "(%s)" s
           else pprintf pc "%s" s
       | <:expr< $int32:s$ >> ->
@@ -1272,8 +1291,12 @@ EXTEND_PRINTER
         <:expr< let $flag:_$ $list:_$ in $_$ >> |
         <:expr< let module $uid:_$ = $_$ in $_$ >> |
         <:expr< match $_$ with [ $list:_$ ] >> |
-        <:expr< try $_$ with [ $list:_$ ] >> as z ->
-          pprintf pc "@[<1>(%q)@]" expr z "" ] ]
+        <:expr< try $_$ with [ $list:_$ ] >> | MLast.ExJdf _ _ _ |
+        MLast.ExRpl _ _ _ | MLast.ExSpw _ _ | MLast.ExPar _ _ _ as z ->
+          pprintf pc "@[<1>(%q)@]" expr z ""
+      | z ->
+          Ploc.raise (MLast.loc_of_expr z)
+            (Failure (sprintf "pr_expr %d" (Obj.tag (Obj.repr z)))) ] ]
   ;
   pr_patt:
     [ "top"
@@ -1679,6 +1702,66 @@ EXTEND_PRINTER
   ;
 END;
 
+value space_before elem pc x = pprintf pc " %p" elem x;
+value or_before elem pc x = pprintf pc "or %p" elem x;
+value amp_after elem pc x = pprintf pc "%p &" elem x;
+
+value option elem pc x =
+  match x with
+  [ Some x -> elem pc x
+  | None -> pprintf pc "" ]
+;
+
+value joinpattern pc (loc, (_, ji), op) =
+  pprintf pc "%s (%p)" (Pcaml.unvala ji) (option patt) (Pcaml.unvala op)
+;
+value joinclause pc (loc, jpl, e) =
+   pprintf pc "@[%p =@]@;%p" (hlistl (amp_after joinpattern) joinpattern)
+     (Pcaml.unvala jpl) expr e
+;
+value joinautomaton pc jc =
+  horiz_vertic
+    (fun () ->
+       pprintf pc "%p" (hlist2 joinclause (or_before joinclause))
+         (Pcaml.unvala jc.MLast.jcVal))
+    (fun () ->
+       pprintf pc "%p" (vlist2 joinclause (or_before joinclause))
+         (Pcaml.unvala jc.MLast.jcVal))
+;
+
+EXTEND_PRINTER
+  pr_str_item:
+    [ [ <:str_item< def $list:jcl$ >> ->
+          pprintf pc "def %p"
+            (hlist2 joinautomaton (and_before joinautomaton)) jcl ] ]
+  ;
+  pr_expr: LEVEL "expr1"
+    [ [ <:expr< def $list:jcl$ in $e$ >> ->
+          horiz_vertic
+            (fun () ->
+               pprintf pc "def %p in %p"
+                 (hlist2 joinautomaton (and_before joinautomaton)) jcl expr e)
+            (fun () ->
+               pprintf pc "@[<a>def %p@ in@]@ %p"
+                 (vlist2 joinautomaton (and_before joinautomaton)) jcl
+                 expr e) ] ]
+  ;
+  pr_expr: LEVEL "apply"
+    [ [ <:expr< reply $opt:eo$ to $s$ >> ->
+          pprintf pc "reply%p in %s" (option (space_before expr)) eo s ] ]
+  ;
+  pr_expr: BEFORE "assign"
+    [ [ <:expr< spawn $e$ >> ->
+          pprintf pc "spawn %p" next e ] ]
+  ;
+  pr_expr: LEVEL "and"
+    [ [ <:expr< $e1$ & $e2$ >> ->
+          horiz_vertic
+            (fun () -> pprintf pc "%p & %p" next e1 curr e2)
+            (fun () -> pprintf pc "%p &@ %p" next e1 curr e2) ] ]
+  ;
+END;
+
 (* main part *)
 
 value sep = Pcaml.inter_phrases;
@@ -1869,7 +1952,7 @@ value class_decl pc ci =
     [ <:class_expr< ($ce$ : $ct$) >> -> (ce, Some ct)
     | ce -> (ce, None) ]
   in
-  let def pc () =
+  let cdef pc () =
     horiz_vertic
       (fun () ->
          pprintf pc "%s%p%s%s%p%p ="
@@ -1890,7 +1973,7 @@ value class_decl pc ci =
          in
          pprintf pc "%p%p =" (plistl patt patt 4) pl class_type_opt ct_opt)
   in
-  pprintf pc "@[%p@;%p@]" def () class_expr ce
+  pprintf pc "@[%p@;%p@]" cdef () class_expr ce
 ;
 
 value variant_decl pc pv =
