@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: pr_r.ml,v 6.74 2012-01-09 14:22:21 deraugla Exp $ *)
+(* $Id: pr_r.ml,v 6.77 2012-03-09 14:01:54 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 #directory ".";
@@ -239,6 +239,13 @@ value only_spaces s =
     else False
 ;
 
+value has_newlines s =
+  loop 0 where rec loop i =
+    if i = String.length s then False
+    else if s.[i] = '\n' then True
+    else loop (i + 1)
+;
+
 value strip_heading_spaces s =
   loop 0 where rec loop i =
     if i = String.length s then ""
@@ -246,19 +253,32 @@ value strip_heading_spaces s =
     else String.sub s i (String.length s - i)
 ;
 
+value strip_one_heading_space s =
+  if String.length s > 0 && s.[0] = ' ' then
+    String.sub s 1 (String.length s - 1)
+  else s
+;
+
 (* expression with adding the possible comment before *)
 value comm_expr expr pc z =
   let loc = MLast.loc_of_expr z in
   let ccc = comm_bef pc loc in
   if ccc = "" then expr pc z
-  else if only_spaces pc.bef then sprintf "%s%s" ccc (expr pc z)
-  else
+  else if only_spaces ccc then
+    sprintf "%s%s" ccc (expr pc z)
+  else if has_newlines ccc then
     expr
       {(pc) with
        bef =
          sprintf "%s%s%s" pc.bef (strip_heading_spaces ccc)
            (String.make (String.length pc.bef) ' ')}
      z
+  else
+    expr
+      {(pc) with
+       bef = sprintf "%s%s" pc.bef (strip_one_heading_space ccc);
+       aft = sprintf "%s%s" pc.aft (Ploc.comment_last loc)}
+      z
 ;
 
 (* couple pattern/anytype with adding the possible comment before *)
@@ -640,10 +660,10 @@ value match_assoc_list pc pwel =
 value rec make_expr_list =
   fun
   [ <:expr< [$x$ :: $y$] >> ->
-      let (xl, c) = make_expr_list y in
-      ([x :: xl], c)
-  | <:expr< [] >> -> ([], None)
-  | x -> ([], Some x) ]
+      let (xl, c, last_comm) = make_expr_list y in
+      ([x :: xl], c, last_comm)
+  | <:expr:< [] >> -> ([], None, Ploc.comment_last loc)
+  | x -> ([], Some x, "") ]
 ;
 
 value rec make_patt_list =
@@ -1324,7 +1344,7 @@ EXTEND_PRINTER
             let el = List.map (fun e -> (e, ";")) el in
             pprintf pc "@[<3>[| %p |]@]" (plist expr 0) el
       | <:expr< [$_$ :: $_$] >> as z ->
-          let (xl, y) = make_expr_list z in
+          let (xl, y, last_comm) = make_expr_list z in
           let xl = List.map (fun x -> (x, ";")) xl in
           match y with
           [ Some y ->
