@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: mLast.mli,v 6.19 2012-01-09 14:22:21 deraugla Exp $ *)
+(* $Id: mLast.mli,v 6.25 2012-03-09 14:54:20 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 #load "pa_macro.cmo";
@@ -99,6 +99,7 @@ and expr =
   | ExFun of loc and V (list (patt * V (option expr) * expr))
   | ExIfe of loc and expr and expr and expr
   | ExInt of loc and V string and string
+  | ExJdf of loc and V (list joinclause) and expr
   | ExLab of loc and V (list (patt * V (option expr)))
   | ExLaz of loc and expr
   | ExLet of loc and V bool and V (list (patt * expr)) and expr
@@ -109,9 +110,12 @@ and expr =
   | ExObj of loc and V (option patt) and V (list class_str_item)
   | ExOlb of loc and patt and V (option expr)
   | ExOvr of loc and V (list (string * expr))
+  | ExPar of loc and expr and expr
   | ExPck of loc and module_expr and option module_type
   | ExRec of loc and V (list (patt * expr)) and option expr
+  | ExRpl of loc and V (option expr) and (loc * V string)
   | ExSeq of loc and V (list expr)
+  | ExSpw of loc and expr
   | ExSnd of loc and expr and V string
   | ExSte of loc and expr and expr
   | ExStr of loc and V string
@@ -166,6 +170,7 @@ and str_item =
   [ StCls of loc and V (list (class_infos class_expr))
   | StClt of loc and V (list (class_infos class_type))
   | StDcl of loc and V (list str_item)
+  | StDef of loc and V (list joinclause)
   | StDir of loc and V string and V (option expr)
   | StExc of loc and V string and V (list ctyp) and V (list string)
   | StExp of loc and expr
@@ -178,6 +183,12 @@ and str_item =
   | StUse of loc and V string and V (list (str_item * loc))
   | StVal of loc and V bool and V (list (patt * expr))
   | StXtr of loc and string and option (V str_item) ]
+and joinclause =
+  { jcLoc : loc;
+    jcVal :
+      V (list
+           (loc * V (list (loc * (loc * V string) * V (option patt))) *
+            expr)) }
 and type_decl =
   { tdNam : V (loc * V string);
     tdPrm : V (list type_var);
