@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: reloc.ml,v 6.21 2012-01-09 14:22:21 deraugla Exp $ *)
+(* $Id: reloc.ml,v 6.26 2012-03-09 14:01:54 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 #load "pa_macro.cmo";
@@ -274,6 +274,9 @@ and expr floc sh =
     | ExInt loc x1 x2 →
         let loc = floc loc in
         ExInt loc x1 x2
+    | ExJdf loc x1 x2 →
+        let loc = floc loc in
+        ExJdf loc (vala_map (List.map (joinclause floc sh)) x1) (self x2)
     | ExLab loc x1 →
         let loc = floc loc in
         ExLab loc
@@ -317,6 +320,9 @@ and expr floc sh =
     | ExOvr loc x1 →
         let loc = floc loc in
         ExOvr loc (vala_map (List.map (fun (x1, x2) → (x1, self x2))) x1)
+    | ExPar loc x1 x2 →
+        let loc = floc loc in
+        ExPar loc (self x1) (self x2)
     | ExPck loc x1 x2 →
         let loc = floc loc in
         ExPck loc (module_expr floc sh x1)
@@ -326,9 +332,16 @@ and expr floc sh =
         ExRec loc
           (vala_map (List.map (fun (x1, x2) → (patt floc sh x1, self x2))) x1)
           (option_map self x2)
+    | ExRpl loc x1 x2 →
+        let loc = floc loc in
+        ExRpl loc (vala_map (option_map self) x1)
+          ((fun (loc, x1) → (floc loc, x1)) x2)
     | ExSeq loc x1 →
         let loc = floc loc in
         ExSeq loc (vala_map (List.map self) x1)
+    | ExSpw loc x1 →
+        let loc = floc loc in
+        ExSpw loc (self x1)
     | ExSnd loc x1 x2 →
         let loc = floc loc in
         ExSnd loc (self x1) x2
@@ -502,6 +515,9 @@ and str_item floc sh =
     | StDcl loc x1 →
         let loc = floc loc in
         StDcl loc (vala_map (List.map self) x1)
+    | StDef loc x1 →
+        let loc = floc loc in
+        StDef loc (vala_map (List.map (joinclause floc sh)) x1)
     | StDir loc x1 x2 →
         let loc = floc loc in
         StDir loc x1 (vala_map (option_map (expr floc sh)) x2)
@@ -544,6 +560,21 @@ and str_item floc sh =
     | StXtr loc x1 x2 →
         let loc = floc loc in
         StXtr loc x1 (option_map (vala_map self) x2) ]
+and joinclause floc sh x =
+  {jcLoc = floc x.jcLoc;
+   jcVal =
+     vala_map
+       (List.map
+          (fun (loc, x1, x2) →
+             (floc loc,
+              vala_map
+                (List.map
+                   (fun (loc, x1, x2) →
+                      (floc loc, (fun (loc, x1) → (floc loc, x1)) x1,
+                       vala_map (option_map (patt floc sh)) x2)))
+                x1,
+              expr floc sh x2)))
+       x.jcVal}
 and type_decl floc sh x =
   {tdNam = vala_map (fun (loc, x1) → (floc loc, x1)) x.tdNam; tdPrm = x.tdPrm;
    tdPrv = x.tdPrv; tdDef = ctyp floc sh x.tdDef;
