@@ -1,9 +1,10 @@
 (* camlp5r *)
-(* $Id: pa_r.ml,v 6.38 2012-01-09 14:22:22 deraugla Exp $ *)
+(* $Id: pa_r.ml,v 6.41 2012-03-09 15:22:46 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 #load "pa_extend.cmo";
 #load "q_MLast.cmo";
+#load "pa_macro.cmo";
 
 open Pcaml;
 
@@ -172,6 +173,8 @@ value warning_deprecated_since_6_00 loc =
   else ()
 ;
 
+(* -- begin copy from pa_r to q_MLast -- *)
+
 EXTEND
   GLOBAL: sig_item str_item ctyp patt expr module_type module_expr
     signature structure class_type class_expr class_sig_item class_str_item
@@ -947,8 +950,55 @@ EXTEND
     [ [ "to" -> True
       | "downto" -> False ] ]
   ;
+  (* -- cut 1 begin -- *)
+  expr: [[]];
 END;
 
+IFDEF JOCAML OR COMPATIBLE_WITH_OLD_OCAML THEN
+  EXTEND
+    GLOBAL: str_item expr;
+    (* -- cut 1 end -- *)
+    str_item: 
+      [ [ "def"; jal = V (LIST1 joinautomaton SEP "and") ->
+            <:str_item< def $_list:jal$ >> ] ]
+    ;
+    expr: LEVEL "top"
+      [ [ "def"; jal = V (LIST1 joinautomaton SEP "and"); "in";
+          e = expr LEVEL "top"->
+            <:expr< def $_list:jal$ in $e$ >> ] ]
+    ;
+    expr: LEVEL "apply"
+      [ [ "reply"; eo = V (OPT expr); "to"; ji = joinident ->
+            <:expr< reply $_opt:eo$ to $jid:ji$ >> ] ]
+    ;
+    expr: BEFORE ":="
+      [ [ "spawn"; e = SELF -> <:expr< spawn $e$ >> ] ]
+    ;
+    expr: LEVEL "&&"
+      [ [ e1 = SELF; "&"; e2 = SELF -> <:expr< $e1$ & $e2$ >> ] ]
+    ;
+    joinautomaton:
+      [ [ jcl = V (LIST1 joinclause SEP "or") ->
+            {MLast.jcLoc = loc; MLast.jcVal = jcl} ] ]
+    ;
+    joinclause:
+      [ [ jpl = V (LIST1 joinpattern SEP "&"); "="; e = expr ->
+            (loc, jpl, e) ] ]
+    ;
+    joinpattern:
+      [ [ ji = joinident; "("; op = V (OPT patt); ")" -> (loc, ji, op) ] ]
+    ;
+    joinident:
+      [ [ i = V LIDENT -> (loc, i) ] ]
+    ;
+    (* -- cut 2 begin -- *)
+    expr: [[]];
+  END;
+END;
+
+(* -- cut 2 end -- *)
+(* -- end copy from pa_r to q_MLast -- *)
+
 value quotation_content s = do {
   loop 0 where rec loop i =
     if i = String.length s then ("", s)
