@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: q_MLast.ml,v 6.35 2012-01-09 14:22:22 deraugla Exp $ *)
+(* $Id: q_MLast.ml,v 6.37 2012-03-09 14:01:54 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2012 *)
 
 #load "pa_extend.cmo";
@@ -314,6 +314,8 @@ value warning_deprecated_since_6_00 loc =
   else ()
 ;
 
+(* -- begin copy from pa_r to q_MLast -- *)
+
 EXTEND
   GLOBAL: sig_item str_item ctyp patt expr module_type module_expr signature
     structure class_type class_expr class_sig_item class_str_item let_binding
@@ -1426,7 +1428,41 @@ EXTEND
     [ [ "to" -> Qast.Bool True
       | "downto" -> Qast.Bool False ] ]
   ;
-  (* Antiquotations for local entries *)
+  str_item:
+    [ [ "def"; jal = SV (LIST1 joinautomaton SEP "and") ->
+          Qast.Node "StDef" [Qast.Loc; jal] ] ]
+  ;
+  expr: LEVEL "top"
+    [ [ "def"; jal = SV (LIST1 joinautomaton SEP "and"); "in";
+        e = expr LEVEL "top" ->
+          Qast.Node "ExJdf" [Qast.Loc; jal; e] ] ]
+  ;
+  expr: LEVEL "apply"
+    [ [ "reply"; eo = SV (OPT expr); "to"; ji = joinident ->
+          Qast.Node "ExRpl" [Qast.Loc; eo; ji] ] ]
+  ;
+  expr: BEFORE ":="
+    [ [ "spawn"; e = SELF -> Qast.Node "ExSpw" [Qast.Loc; e] ] ]
+  ;
+  expr: LEVEL "&&"
+    [ [ e1 = SELF; "&"; e2 = SELF -> Qast.Node "ExPar" [Qast.Loc; e1; e2] ] ]
+  ;
+  joinautomaton:
+    [ [ jcl = SV (LIST1 joinclause SEP "or") ->
+          Qast.Record [("jcLoc", Qast.Loc); ("jcVal", jcl)] ] ]
+  ;
+  joinclause:
+    [ [ jpl = SV (LIST1 joinpattern SEP "&"); "="; e = expr ->
+          Qast.Tuple [Qast.Loc; jpl; e] ] ]
+  ;
+  joinpattern:
+    [ [ ji = joinident; "("; op = SV (OPT patt); ")" ->
+          Qast.Tuple [Qast.Loc; ji; op] ] ]
+  ;
+  joinident:
+    [ [ i = SV LIDENT -> Qast.Tuple [Qast.Loc; i] ] ]
+  ;
+  (* -- end copy from pa_r to q_MLast -- *)
   a_ti:
     [ [ "~"; a = ANTIQUOT -> Qast.VaAnt "~" loc a ] ]
   ;
@@ -1439,6 +1475,9 @@ EXTEND
   a_qic:
     [ [ "?"; a = ANTIQUOT; ":" -> Qast.VaAnt "?" loc a ] ]
   ;
+  joinident:
+    [ [ a = ANTIQUOT "jid" -> Qast.VaAnt "jid" loc a ] ]
+  ;
 END;
 
 (* Antiquotations *)
